package controllers

import (
	"ERP_storage/Nats/NatsServer"
	"ERP_storage/conf"
	"ERP_storage/models/Account"
	"ERP_storage/models/Contract"
	"ERP_storage/models/Stock"
	"fmt"
	userlibs "git.baozhida.cn/ERP_libs/User"
	"git.baozhida.cn/ERP_libs/lib"
	"github.com/beego/beego/v2/adapter/orm"
	beego "github.com/beego/beego/v2/server/web"
	"math"
	"strconv"
	"strings"
)

type ContractController struct {
	beego.Controller
	User userlibs.User
}

func (c *ContractController) Prepare() {
	c.User = *Account.User_r
}

func (c *ContractController) Contract_List() {

	// 分页参数 初始化
	page, _ := c.GetInt("page")
	if page < 1 {
		page = 1
	}
	page_z, _ := c.GetInt("page_z")
	if page_z < 1 {
		page_z = conf.Page_size
	}

	// 查询
	T_name := c.GetString("T_name")
	T_state, _ := c.GetInt("T_state")

	userList, _ := NatsServer.Read_User_List_All()
	Account.Read_User_All_Map(userList)

	ContractDao := Contract.NewContract(orm.NewOrm())
	R_List, R_cnt := ContractDao.Read_Contract_List("", T_name, T_state, 0, page, page_z)

	var r_jsons lib.R_JSONS
	r_jsons.Num = R_cnt
	r_jsons.Data = R_List
	r_jsons.Page = page
	r_jsons.Page_size = int(math.Ceil(float64(R_cnt) / float64(page_z)))

	c.Data["json"] = lib.JSONS{Code: 200, Msg: "ok!", Data: r_jsons}
	c.ServeJSON()
	return
}

func (c *ContractController) Contract_User_List() {

	// 分页参数 初始化
	page, _ := c.GetInt("page")
	if page < 1 {
		page = 1
	}
	page_z, _ := c.GetInt("page_z")
	if page_z < 1 {
		page_z = conf.Page_size
	}

	// 查询
	T_name := c.GetString("T_name")
	T_state, _ := c.GetInt("T_state")

	userList, _ := NatsServer.Read_User_List_All()
	Account.Read_User_All_Map(userList)

	ContractDao := Contract.NewContract(orm.NewOrm())
	R_List, R_cnt := ContractDao.Read_Contract_List(c.User.T_uuid, T_name, T_state, 0, page, page_z)

	var r_jsons lib.R_JSONS
	r_jsons.Num = R_cnt
	r_jsons.Data = R_List
	r_jsons.Page = page
	r_jsons.Page_size = int(math.Ceil(float64(R_cnt) / float64(page_z)))

	c.Data["json"] = lib.JSONS{Code: 200, Msg: "ok!", Data: r_jsons}
	c.ServeJSON()
	return
}

func (c *ContractController) Contract_Get() {

	// 查询
	T_number := c.GetString("T_number")

	o := orm.NewOrm()
	ContractDao := Contract.NewContract(o)
	DeviceDao := Stock.NewDevice(o)
	ContractProductDao := Contract.NewContractProduct(o)
	contract, err := ContractDao.Read_Contract_ByT_number(T_number)
	if err != nil {
		c.Data["json"] = lib.JSONS{Code: 202, Msg: "查询失败!"}
		c.ServeJSON()
		return
	}

	productList := ContractProductDao.Read_ContractProduct_List(contract.T_number)

	var pList []Contract.ContractProduct_R
	for _, v := range productList {
		p := Contract.ContractProductToContractProduct_R(v)
		p.T_device_list, _ = DeviceDao.Read_DeviceSn_List(T_number, v.T_product_id, "", "")
		pList = append(pList, p)
	}

	userList, _ := NatsServer.Read_User_List_All()
	Account.Read_User_All_Map(userList)

	c.Data["json"] = lib.JSONS{Code: 200, Msg: "ok!", Data: Contract.ContractToContract_Detail(contract, pList)}
	c.ServeJSON()
	return
}

func (c *ContractController) Contract_Add() {

	T_number := c.GetString("T_number")
	T_customer := c.GetString("T_customer")
	T_money, _ := c.GetFloat("T_money")
	T_type, _ := c.GetInt("T_type")
	T_date := c.GetString("T_date")
	T_product := c.GetString("T_product")
	T_remark := c.GetString("T_remark")
	T_pdf := c.GetString("T_pdf")

	var_ := Contract.Contract{
		T_number:   T_number,
		T_customer: T_customer,
		T_money:    float32(T_money),
		T_type:     T_type,
		T_date:     T_date,
		T_out:      1,
		T_State:    3,
		T_remark:   T_remark,
		T_pdf:      T_pdf,
		T_submit:   c.User.T_uuid,
	}

	o := orm.NewOrm()
	o.Begin()
	ContractDao := Contract.NewContract(o)
	ContractProductDao := Contract.NewContractProduct(o)

	_, err := ContractDao.Read_Contract_ByT_number(T_number)
	if err == nil {
		o.Rollback()
		c.Data["json"] = lib.JSONS{Code: 202, Msg: "合同编号重复！"}
		c.ServeJSON()
		return
	}

	Id, err := ContractDao.Add_Contract(var_)
	if err != nil {
		o.Rollback()
		c.Data["json"] = lib.JSONS{Code: 203, Msg: "添加失败"}
		c.ServeJSON()
		return
	}

	//合同类型 1-销售合同 2-验证合同
	if T_type == 2 {
		NatsServer.AddUserLogs(c.User.T_uuid, "合同", "添加", var_)

		c.Data["json"] = lib.JSONS{Code: 200, Msg: "ok!", Data: Id}
		c.ServeJSON()
		return
	}

	if len(T_product) == 0 {
		o.Rollback()
		c.Data["json"] = lib.JSONS{Code: 202, Msg: "销售合同产品明细不能为空"}
		c.ServeJSON()
		return
	}
	productList := lib.SplitString(T_product, "|")
	for _, v := range productList {
		product_id, _ := strconv.Atoi(strings.Split(v, ",")[0])
		num, _ := strconv.Atoi(strings.Split(v, ",")[1])
		contractProduct := Contract.ContractProduct{
			T_contract_number: T_number,
			T_product_id:      product_id,
			T_product_total:   num,
			T_product_out:     0, // 已出库数量
			T_State:           1, // 1-未出库 2-已部分出库 3-已全部出库
		}
		_, err = ContractProductDao.Add_ContractProduct(contractProduct)
		if err != nil {
			o.Rollback()
			c.Data["json"] = lib.JSONS{Code: 203, Msg: "添加失败"}
			c.ServeJSON()
			return
		}

	}

	o.Commit()

	NatsServer.AddUserLogs(c.User.T_uuid, "合同", "添加", var_)

	c.Data["json"] = lib.JSONS{Code: 200, Msg: "ok!", Data: T_number}
	c.ServeJSON()
	return
}

func (c *ContractController) Contract_Approval() {

	T_number := c.GetString("T_number")
	T_state, _ := c.GetInt("T_state")

	o := orm.NewOrm()
	ContractDao := Contract.NewContract(o)

	contract, err := ContractDao.Read_Contract_ByT_number(T_number)
	if err != nil {
		o.Rollback()
		c.Data["json"] = lib.JSONS{Code: 202, Msg: "修改失败！"}
		c.ServeJSON()
		return
	}
	// 1-已通过 2-未通过
	contract.T_State = T_state
	contract.T_approver = c.User.T_uuid
	err = ContractDao.Update_Contract(contract, "T_State", "T_approver")
	if err != nil {
		o.Rollback()
		c.Data["json"] = lib.JSONS{Code: 203, Msg: "添加失败"}
		c.ServeJSON()
		return
	}
	if T_state == 1 {
		NatsServer.AddNews(contract.T_submit, fmt.Sprintf("您提交的合同(%s)【审核已通过】", T_number), conf.ContractApprovalUrl)
	}
	if T_state == 2 {
		NatsServer.AddNews(contract.T_submit, fmt.Sprintf("您提交的合同(%s)【审核未通过】", T_number), "")
	}
	NatsServer.AddUserLogs(c.User.T_uuid, "合同", "审核", contract)

	c.Data["json"] = lib.JSONS{Code: 200, Msg: "ok!", Data: T_number}
	c.ServeJSON()
	return
}
func (c *ContractController) Contract_Edit() {
	T_number := c.GetString("T_number")
	T_customer := c.GetString("T_customer")
	T_money, _ := c.GetFloat("T_money")
	T_date := c.GetString("T_date")
	T_product := c.GetString("T_product")
	T_remark := c.GetString("T_remark")
	T_pdf := c.GetString("T_pdf")

	o := orm.NewOrm()
	o.Begin()
	ContractDao := Contract.NewContract(o)
	ContractProductDao := Contract.NewContractProduct(o)

	contract, err := ContractDao.Read_Contract_ByT_number(T_number)
	if err != nil {
		o.Rollback()
		c.Data["json"] = lib.JSONS{Code: 202, Msg: "修改失败！"}
		c.ServeJSON()
		return
	}

	if len(T_customer) > 0 {
		contract.T_customer = T_customer
	}
	if T_money > 0 {
		contract.T_money = float32(T_money)
	}
	if len(T_date) > 0 {
		contract.T_date = T_date
	}
	if len(T_remark) > 0 {
		contract.T_remark = T_remark
	}
	if len(T_pdf) > 0 {
		contract.T_pdf = T_pdf
	}

	//1-已通过 2-未通过 3-待审核 合同状态为未通过，修改之后将状态更改为待审核
	if contract.T_State == 2 {
		contract.T_State = 3
	}

	err = ContractDao.Update_Contract(contract, "T_customer", "T_money", "T_date", "T_remark", "T_pdf")
	if err != nil {
		o.Rollback()
		c.Data["json"] = lib.JSONS{Code: 203, Msg: "添加失败"}
		c.ServeJSON()
		return
	}

	//合同类型 1-销售合同 2-验证合同
	if contract.T_type == 2 {
		NatsServer.AddUserLogs(c.User.T_uuid, "合同", "修改", contract)

		c.Data["json"] = lib.JSONS{Code: 200, Msg: "ok!", Data: contract.Id}
		c.ServeJSON()
		return
	}

	if len(T_product) == 0 {
		o.Rollback()
		c.Data["json"] = lib.JSONS{Code: 202, Msg: "销售合同产品明细不能为空"}
		c.ServeJSON()
		return
	}
	productList := lib.SplitString(T_product, "|")
	for _, v := range productList {
		product_id, _ := strconv.Atoi(strings.Split(v, ",")[0])
		num, _ := strconv.Atoi(strings.Split(v, ",")[1])

		var cp Contract.ContractProduct
		cp, err = ContractProductDao.Read_ContractProduct_ByT_number_T_product_id(T_number, product_id)
		if err != nil {
			if err.Error() == orm.ErrNoRows.Error() {
				contractProduct := Contract.ContractProduct{
					T_contract_number: T_number,
					T_product_id:      product_id,
					T_product_total:   num,
					T_product_out:     0, // 已出库数量
					T_State:           1, // 1-未出库 2-已部分出库 3-已全部出库
				}
				_, err = ContractProductDao.Add_ContractProduct(contractProduct)
				if err != nil {
					o.Rollback()
					c.Data["json"] = lib.JSONS{Code: 203, Msg: "修改失败"}
					c.ServeJSON()
					return
				}

			}
			o.Rollback()
			c.Data["json"] = lib.JSONS{Code: 203, Msg: "修改失败"}
			c.ServeJSON()
			return
		}
		if cp.T_product_total == num {
			continue
		}
		cp.T_product_total = num
		err = ContractProductDao.Update_ContractProduct(cp, "T_product_total")
		if err != nil {
			o.Rollback()
			c.Data["json"] = lib.JSONS{Code: 203, Msg: "修改失败"}
			c.ServeJSON()
			return
		}

	}

	o.Commit()

	NatsServer.AddUserLogs(c.User.T_uuid, "合同", "修改", contract)

	c.Data["json"] = lib.JSONS{Code: 200, Msg: "ok!", Data: contract.Id}
	c.ServeJSON()
	return
}
func (c *ContractController) Contract_Del() {

	T_number := c.GetString("T_number")

	o := orm.NewOrm()
	o.Begin()
	ContractDao := Contract.NewContract(o)
	ContractProductDao := Contract.NewContractProduct(o)

	contract, err := ContractDao.Read_Contract_ByT_number(T_number)
	if err != nil {
		o.Rollback()
		c.Data["json"] = lib.JSONS{Code: 202, Msg: "删除失败！"}
		c.ServeJSON()
		return
	}
	//1-已通过 2-未通过 3-待审核 审核状态
	if contract.T_State == 1 {
		o.Rollback()
		c.Data["json"] = lib.JSONS{Code: 202, Msg: "合同已审核通过，禁止删除！"}
		c.ServeJSON()
		return
	}

	err = ContractDao.Delete_Contract(contract)
	if err != nil {
		o.Rollback()
		c.Data["json"] = lib.JSONS{Code: 202, Msg: "T_number Err!"}
		c.ServeJSON()
		return
	}
	//合同类型 1-销售合同 2-验证合同
	if contract.T_type == 1 {
		cpList := ContractProductDao.Read_ContractProduct_List(T_number)
		for _, v := range cpList {
			err = ContractProductDao.Delete_ContractProduct(v)
			if err != nil {
				o.Rollback()
				c.Data["json"] = lib.JSONS{Code: 202, Msg: "删除失败!"}
				c.ServeJSON()
				return
			}
		}
	}
	o.Commit()

	NatsServer.AddUserLogs(c.User.T_uuid, "合同", "删除", T_number)

	c.Data["json"] = lib.JSONS{Code: 200, Msg: "ok!", Data: T_number}
	c.ServeJSON()
	return
}

func (c *ContractController) Contract_List_For_Out() {

	// 分页参数 初始化
	page, _ := c.GetInt("page")
	if page < 1 {
		page = 1
	}
	page_z, _ := c.GetInt("page_z")
	if page_z < 1 {
		page_z = conf.Page_size
	}

	// 查询
	T_name := c.GetString("T_name")

	userList, _ := NatsServer.Read_User_List_All()
	Account.Read_User_All_Map(userList)

	ContractDao := Contract.NewContract(orm.NewOrm())
	R_List, R_cnt := ContractDao.Read_Contract_List("", T_name, 1, 1, page, page_z)

	var r_jsons lib.R_JSONS
	r_jsons.Num = R_cnt
	r_jsons.Data = R_List
	r_jsons.Page = page
	r_jsons.Page_size = int(math.Ceil(float64(R_cnt) / float64(page_z)))

	c.Data["json"] = lib.JSONS{Code: 200, Msg: "ok!", Data: r_jsons}
	c.ServeJSON()
	return
}

// 合同下的产品列表 出库时使用
func (c *ContractController) Contract_Product_List() {

	// 查询
	T_number := c.GetString("T_number")
	T_depot_id, _ := c.GetInt("T_depot_id")

	o := orm.NewOrm()
	ContractDao := Contract.NewContract(o)
	ContractProductDao := Contract.NewContractProduct(o)
	contract, err := ContractDao.Read_Contract_ByT_number(T_number)
	if err != nil {
		c.Data["json"] = lib.JSONS{Code: 202, Msg: "查询失败!"}
		c.ServeJSON()
		return
	}
	if contract.T_type == 2 {
		c.Data["json"] = lib.JSONS{Code: 202, Msg: "验证合同无产品列表!"}
		c.ServeJSON()
		return
	}

	productList := ContractProductDao.Read_ContractProduct_List(contract.T_number)
	Stock.Read_Stock_All_Map(T_depot_id)

	var pList []Contract.ContractProduct_Out
	for _, v := range productList {
		p := Contract.ContractProductToContractProduct_Out(v)
		p.T_stock_total = Stock.Read_Stock_Get(T_depot_id, p.T_product_id)
		pList = append(pList, p)
	}

	c.Data["json"] = lib.JSONS{Code: 200, Msg: "ok!", Data: pList}
	c.ServeJSON()
	return
}
