package Contract

import (
	"ERP_storage/logs"
	"ERP_storage/models/Account"
	"git.baozhida.cn/ERP_libs/lib"
	orm2 "github.com/beego/beego/v2/client/orm"
	"time"

	_ "github.com/astaxie/beego/cache/redis"
	"github.com/beego/beego/v2/adapter/orm"
	_ "github.com/go-sql-driver/mysql"
)

// 合同
type Contract struct {
	Id         int     `orm:"column(ID);size(11);auto;pk"`
	T_number   string  `orm:"size(256);null"`         // 合同编号
	T_customer string  `orm:"size(256);null"`         // 客户名称
	T_money    float32 `orm:"digits(12);decimals(2)"` // 合同金额
	T_type     int     `orm:"size(2);default(1)"`     // 合同类型 1-销售合同 2-验证合同
	T_date     string  `orm:"size(256);null"`         // 业务日期
	T_out      int     `orm:"size(2);default(1)"`     // 出库状态  1-未出库 2-已部分出库 3-已全部出库
	T_State    int     `orm:"size(2);default(1)"`     // 0 删除（伪删除）  1-已通过 2-未通过 3-待审核 审核状态
	T_pdf      string  `orm:"size(256);null"`         // 附件
	T_remark   string  `orm:"type(text);null"`        // 备注

	T_submit   string `orm:"size(256);null"` // 合同提交人员
	T_approver string `orm:"size(256);null"` // 审批人

	CreateTime time.Time `orm:"column(create_time);type(timestamp);null;auto_now_add"` //auto_now_add 第一次保存时才设置时间
	UpdateTime time.Time `orm:"column(update_time);type(timestamp);null;auto_now"`     //auto_now 每次 model 保存时都会对时间自动更新
}

func (t *Contract) TableName() string {
	return "contract" // 数据库名称   // ************** 替换 FormulaList **************
}

type ContractDaoImpl struct {
	orm orm.Ormer
}

func NewContract(orm orm.Ormer) *ContractDaoImpl {
	return &ContractDaoImpl{orm: orm}
}

func init() {
	//注册模型
	orm.RegisterModel(new(Contract))
}

type Contract_R struct {
	Id              int
	T_number        string  // 合同编号
	T_customer      string  // 客户名称
	T_money         float32 // 合同金额
	T_type          int     // 合同类型 1-销售合同 2-验证合同
	T_date          string  // 业务日期
	T_out           int     // 出库状态  1-未出库 2-已部分出库 3-已全部出库
	T_submit        string  // 合同提交人员uuid
	T_submit_name   string  // 合同提交人员名称
	T_approver      string  // 合同审批人员uuid
	T_approver_name string  // 合同审批人员名称
	T_pdf           string  // 附件
	T_State         int     // 1-已通过 2-未通过 3-待审核 审核状态
}

type Contract_Detail struct {
	Id              int
	T_number        string
	T_customer      string
	T_money         float32
	T_type          int
	T_date          string
	T_submit        string
	T_submit_name   string
	T_approver      string // 审批人uuid
	T_approver_name string // 审批人名称
	T_out           int
	T_State         int
	T_remark        string
	T_pdf           string
	T_Product       []ContractProduct_R
}

func ContractToContract_R(t Contract) (r Contract_R) {
	// 1-未出库 2-已部分出库 3-已全部出库
	r.T_out = t.T_out
	// 1-已通过 2-未通过 3-待审核
	//if t.T_State == 1 {
	//	r.T_out = 2
	//	state := Read_ContractProduct_T_State_List(t.T_number)
	//	if len(state) == 1 {
	//		if state[0] == 1 {
	//			r.T_out = 1
	//		}
	//		if state[0] == 3 {
	//			r.T_out = 3
	//		}
	//	}
	//}
	r.Id = t.Id
	r.T_number = t.T_number
	r.T_customer = t.T_customer
	r.T_money = t.T_money
	r.T_type = t.T_type
	r.T_date = t.T_date
	r.T_State = t.T_State
	r.T_pdf = t.T_pdf
	r.T_submit = t.T_submit
	r.T_submit_name = Account.Read_User_T_name_Get(t.T_submit)
	r.T_approver = t.T_approver
	r.T_approver_name = Account.Read_User_T_name_Get(t.T_approver)
	return r
}

func ContractToContract_Detail(t Contract, productList []ContractProduct_R) (r Contract_Detail) {
	// 1-未出库 2-已部分出库 3-已全部出库
	r.T_out = t.T_out
	// 1-已通过 2-未通过 3-待审核
	//if t.T_State == 1 {
	//	r.T_out = 2
	//	state := Read_ContractProduct_T_State_List(t.T_number)
	//	if len(state) == 1 {
	//		if state[0] == 1 {
	//			r.T_out = 1
	//		}
	//		if state[0] == 3 {
	//			r.T_out = 3
	//		}
	//	}
	//}

	r.Id = t.Id
	r.T_number = t.T_number
	r.T_customer = t.T_customer
	r.T_money = t.T_money
	r.T_type = t.T_type
	r.T_date = t.T_date
	r.T_State = t.T_State
	r.T_remark = t.T_remark
	r.T_pdf = t.T_pdf
	r.T_submit = t.T_submit
	r.T_submit_name = Account.Read_User_T_name_Get(t.T_submit)
	r.T_approver = t.T_approver
	r.T_approver_name = Account.Read_User_T_name_Get(t.T_approver)
	r.T_Product = productList

	return r
}

// 添加
func (dao *ContractDaoImpl) Add_Contract(r Contract) (id int64, err error) {
	id, err = dao.orm.Insert(&r)
	if err != nil {
		logs.Error(lib.FuncName(), err)
	}
	return id, err
}

// 获取 ById
func (dao *ContractDaoImpl) Read_Contract_ByT_number(T_number string) (r Contract, err error) {
	qs := dao.orm.QueryTable(new(Contract))

	err = qs.Filter("T_number", T_number).Filter("T_State__gt", 0).One(&r)
	if err != nil {
		logs.Error(lib.FuncName(), err)
	}
	return
}

func (dao *ContractDaoImpl) Read_Contract_ById(Id int) (r Contract, err error) {
	qs := dao.orm.QueryTable(new(Contract))
	err = qs.Filter("Id", Id).One(&r)
	if err != nil {
		logs.Error(lib.FuncName(), err)
	}
	return
}

// 修改
func (dao *ContractDaoImpl) Update_Contract(m Contract, cols ...string) error {
	_, err := dao.orm.Update(&m, cols...)
	if err != nil {
		logs.Error(lib.FuncName(), err)
		return err
	}
	return nil
}

// 删除
func (dao *ContractDaoImpl) Delete_Contract(v Contract) error {
	v.T_State = 0
	_, err := dao.orm.Update(&v, "T_State")
	if err != nil {
		logs.Error(lib.FuncName(), err)
	}
	return err
}

// 获取列表
func (dao *ContractDaoImpl) Read_Contract_List(T_submit, T_name string, T_state, T_out, page, page_z int) (r_ []Contract_R, cnt int64) {
	// 也可以直接使用 Model 结构体作为表名
	qs := dao.orm.QueryTable(new(Contract))
	var offset int64
	if page <= 1 {
		offset = 0
	} else {
		offset = int64((page - 1) * page_z)
	}

	// 过滤
	cond := orm.NewCondition()
	cond1 := cond.And("T_State__gt", 0)
	if T_state > 0 {
		cond1 = cond.And("T_State", T_state)
	}

	if len(T_name) > 0 {
		cond1 = cond1.AndCond(cond.Or("T_number__icontains", T_name).Or("T_sn__icontains", T_name))
	}

	if len(T_submit) > 0 {
		cond1 = cond1.And("T_submit", T_submit)
	}

	// T_out == 1 查询未出库或部分出库数据
	if T_out == 1 {
		cond1 = cond1.AndNot("T_out", 3)

	}

	// 查询
	var r []Contract
	_, err := qs.Limit(page_z, offset).SetCond((*orm2.Condition)(cond1)).OrderBy("-Id").All(&r)

	if err != nil {
		logs.Error(lib.FuncName(), err)
		return
	}

	cnt, err = qs.SetCond((*orm2.Condition)(cond1)).Count()
	if err != nil {
		logs.Error(lib.FuncName(), err)
		return
	}

	for _, v := range r {
		r_ = append(r_, ContractToContract_R(v))
	}

	return r_, cnt
}
