package Stock

import (
	"ERP_storage/logs"
	"fmt"
	"git.baozhida.cn/ERP_libs/lib"
	_ "github.com/astaxie/beego/cache/redis"
	"github.com/beego/beego/v2/adapter/orm"
	orm2 "github.com/beego/beego/v2/client/orm"
	"time"
)

type Device struct {
	Id                int       `orm:"column(ID);size(11);auto;pk"`
	T_contract_number string    `orm:"size(256);null"`                                        // 合同编号
	T_product_id      int       `orm:"size(20);null"`                                         // 产品id
	T_in_number       string    `orm:"size(256);null"`                                        // 入库编号
	T_out_number      string    `orm:"size(256);null"`                                        // 出库编号
	T_sn              string    `orm:"size(256);null"`                                        // 设备sn
	T_iccid           string    `orm:"size(256);null"`                                        // sim卡号
	T_imei            string    `orm:"size(256);null"`                                        // 模组imei
	T_State           int       `orm:"size(2);default(2)"`                                    // 1-已出库 2-未出库
	T_remark          string    `orm:"type(text);null"`                                       // 备注
	CreateTime        time.Time `orm:"column(create_time);type(timestamp);null;auto_now_add"` //auto_now_add 第一次保存时才设置时间
	UpdateTime        time.Time `orm:"column(update_time);type(timestamp);null;auto_now"`     //auto_now 每次 model 保存时都会对时间自动更新
}

func (t *Device) TableName() string {
	return "device" // 数据库名称   // ************** 替换 FormulaList **************
}

type DeviceDaoImpl struct {
	orm orm.Ormer
}

func NewDevice(orm orm.Ormer) *DeviceDaoImpl {
	return &DeviceDaoImpl{orm: orm}
}

func init() {
	//注册模型
	orm.RegisterModel(new(Device))
}

type Device_R struct {
	Id                int
	T_contract_number string // 合同编号
	T_product_id      int    // 产品id
	T_in_number       string // 入库编号
	T_out_number      string // 出库编号
	T_sn              string // 设备sn
	T_iccid           string
	T_imei            string // 物联网卡号
	T_State           int    // 1-已出库 2-未出库
}

func DeviceToDevice_R(t Device) (r Device_R) {
	r.Id = t.Id
	r.T_contract_number = t.T_contract_number
	r.T_product_id = t.T_product_id
	r.T_in_number = t.T_in_number
	r.T_out_number = t.T_out_number
	r.T_sn = t.T_sn
	r.T_iccid = t.T_iccid
	r.T_imei = t.T_imei
	r.T_State = t.T_State
	return r
}

// 添加
func (dao *DeviceDaoImpl) Add_Device(r Device) (id int64, err error) {
	id, err = dao.orm.Insert(&r)
	if err != nil {
		logs.Error(lib.FuncName(), err)
	}
	return id, err
}

// T_type 1-出库 2-入库
func (dao *DeviceDaoImpl) AddOrUpdate_Device(r Device, T_State int) (id int64, err error) {

	qs := dao.orm.QueryTable(new(Device))
	var device Device
	err = qs.Filter("T_sn", r.T_sn).One(&device)
	if err != nil {
		if err.Error() == orm.ErrNoRows.Error() {
			id, err = dao.orm.Insert(&r)
			if err != nil {
				logs.Error(lib.FuncName(), err)
			}
			return
		} else {
			logs.Error(lib.FuncName(), err)
			return
		}
	}

	// 1-已出库 2-未出库
	device.T_State = T_State
	T_remark := ""
	if T_State == 1 {
		T_remark = fmt.Sprintf("%s:%s(%s)|", time.Now().Format("2006-01-02"), "出库", device.T_out_number)
	}
	if T_State == 2 {
		T_remark = fmt.Sprintf("%s:%s(%s)|", time.Now().Format("2006-01-02"), "入库", device.T_in_number)
	}
	device.T_remark = device.T_remark + T_remark

	_, err = dao.orm.Update(&device, "T_in_number", "T_out_number", "T_State", "T_remark")
	if err != nil {
		logs.Error(lib.FuncName(), err)
		return
	}

	return
}

func (dao *DeviceDaoImpl) Read_Device_ByT_sn(T_sn string) (r Stock, err error) {
	qs := dao.orm.QueryTable(new(Stock))
	err = qs.Filter("T_sn", T_sn).One(&r)
	if err != nil {
		logs.Error(lib.FuncName(), err)
	}
	return
}

func (dao *DeviceDaoImpl) Read_DeviceSn_List(T_contract_number string, T_product_id int, T_in_number, T_out_number string) (r []string, err error) {
	qs := dao.orm.QueryTable(new(Device))
	// 过滤
	cond := orm.NewCondition()
	cond.And("T_product_id", T_product_id)
	if len(T_contract_number) > 0 {
		cond = cond.And("T_contract_number", T_contract_number)
	}

	if len(T_in_number) > 0 {
		cond = cond.And("T_in_number", T_in_number)
	}
	if len(T_out_number) > 0 {
		cond = cond.And("T_out_number", T_out_number)
	}
	var maps []Device
	_, err = qs.SetCond((*orm2.Condition)(cond)).All(&maps)
	if err != nil {
		logs.Error(lib.FuncName(), err)
		return
	}

	for _, v := range maps {
		r = append(r, v.T_sn)
	}

	return
}
